function ir = ir_generic(X,phi,x0,d,sofa,conf)
%IR_GENERIC Generate a IR
%
%   Usage: ir = ir_generic(X,phi,x0,d,sofa,conf)
%
%   Input parameters:
%       X       - listener position / m
%       phi     - listener direction [head orientation] / rad
%                 0 means the head is oriented towards the x-axis.
%       x0      - secondary sources [n x 7] / m
%       d       - driving signals [m x n]
%       sofa    - impulse response data set for the secondary sources
%       conf    - configuration struct (see SFS_config)
%
%   Output parameters:
%       ir      - Impulse response for the desired driving functions (nx2 matrix)
%
%   IR_GENERIC(X,phi,x0,d,sofa,conf) calculates a binaural room impulse
%   response for the given secondary sources and driving signals.
%
%   See also: ir_wfs, ir_nfchoa, ir_point_source, auralize_ir

%*****************************************************************************
% Copyright (c) 2010-2016 Quality & Usability Lab, together with             *
%                         Assessment of IP-based Applications                *
%                         Telekom Innovation Laboratories, TU Berlin         *
%                         Ernst-Reuter-Platz 7, 10587 Berlin, Germany        *
%                                                                            *
% Copyright (c) 2013-2016 Institut fuer Nachrichtentechnik                   *
%                         Universitaet Rostock                               *
%                         Richard-Wagner-Strasse 31, 18119 Rostock           *
%                                                                            *
% This file is part of the Sound Field Synthesis-Toolbox (SFS).              *
%                                                                            *
% The SFS is free software:  you can redistribute it and/or modify it  under *
% the terms of the  GNU  General  Public  License  as published by the  Free *
% Software Foundation, either version 3 of the License,  or (at your option) *
% any later version.                                                         *
%                                                                            *
% The SFS is distributed in the hope that it will be useful, but WITHOUT ANY *
% WARRANTY;  without even the implied warranty of MERCHANTABILITY or FITNESS *
% FOR A PARTICULAR PURPOSE.                                                  *
% See the GNU General Public License for more details.                       *
%                                                                            *
% You should  have received a copy  of the GNU General Public License  along *
% with this program.  If not, see <http://www.gnu.org/licenses/>.            *
%                                                                            *
% The SFS is a toolbox for Matlab/Octave to  simulate and  investigate sound *
% field  synthesis  methods  like  wave  field  synthesis  or  higher  order *
% ambisonics.                                                                *
%                                                                            *
% http://github.com/sfstoolbox/sfs                      sfstoolbox@gmail.com *
%*****************************************************************************


%% ===== Checking of input  parameters ==================================
nargmin = 6;
nargmax = 6;
narginchk(nargmin,nargmax);
isargposition(X);
isargscalar(phi);
isargsecondarysource(x0);
isargmatrix(d);
isargstruct(conf);
if size(x0,1)~=size(d,2)
    error(['%s: The number of secondary sources (%i) and driving ', ...
        'signals (%i) does not correspond.'], ...
        upper(mfilename),size(x0,1),size(d,2));
end

%% ===== Configuration ==================================================
N = conf.N; % target length of BRS impulse responses


%% ===== Variables ======================================================
phi = correct_azimuth(phi);


%% ===== BRIR ===========================================================
% Initial values
ir_generic = zeros(N,2);

% Create a BRIR for every single loudspeaker
warning('off','SFS:irs_intpol');
for ii=1:size(x0,1)

    % === Get the desired impulse response.
    % If needed interpolate the given impulse response set and weight, delay the
    % impulse for the correct distance
    ir = get_ir(sofa,X,[phi 0],x0(ii,1:3),'cartesian',conf);

    % === Sum up virtual loudspeakers/HRIRs and add loudspeaker time delay ===
    % Also applying the weights of the secondary sources including integration
    % weights or tapering windows etc.
    ir_generic = ir_generic + fix_length(convolution(ir,d(:,ii)),N).*x0(ii,7);

end
warning('on','SFS:irs_intpol');


%% ===== Headphone compensation =========================================
ir = compensate_headphone(ir_generic,conf);
